#include "enemies.h"
#include "shot.h"

const int MaxFireTime = 100;
const double ShotSpeed = 0.1;
static const double TurnRate = 1.5;
static const double SaucerTurnRate = 3.0;
static const double AzimuthTolerance = 1.0;

Turret::Turret(Manager &m, MapBlock::MapItemType t) : Enemy(m), type(t) 
{
	location.z = TurretHeight;
	collidable = true;
	fireTimer = MaxFireTime;
	fireSpeed = MaxFireTime;
	radius = 0.4;
	if (type == MapBlock::TurretItem1) {
		hitPoints = 50;
	}
}
void Turret::setLocation(const Vect3d &where)
{ 
	location = where; 
	location.z = TurretHeight;
	switch (type) {
		case MapBlock::TurretItem2 :
			if (where.y > 4.5) {
				azimuth = 225;
			} else {
				azimuth = 315;
			}
			break;
		default :
			azimuth = 270;
			break;
	}
	desiredAzimuth = azimuth;
}

			
void Turret::render()
{
	glEnable(GL_LIGHTING);
	glDisable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glColor3f(1,1,1);
	manager.rm.turrettexture.Bind();
	// manager.rm.saucertexture.Bind();
	glPushMatrix();
	float scale = 1.0 / 16;
	glTranslatef(location.x, location.y, location.z);
	glScalef(scale,scale,scale);
	glRotatef(- azimuth, 0.0f, 0.0f, 1.0f);
	manager.rm.turretmodel.draw(true);
	glPopMatrix();
	drawHurtFlash();
}

void Turret::runLogic()
{
	checkBounds();
	if (fireTimer > 0) {
		fireTimer --;
		if ((type == MapBlock::TurretItem3) && (fireTimer == (fireSpeed / 2)))
		{
			// aim gun.
			Vect3d rel = manager.playerLocation - location;
			desiredAzimuth = rel.angle();
		}
	} else {
		doFire();
	}
	// Rotate the turret.
	double azimuthDiff = desiredAzimuth - azimuth;
	normaliseAngle(azimuthDiff);
	if (fabs(azimuthDiff) < AzimuthTolerance) {
		azimuth = desiredAzimuth;
	} else {
		if (azimuthDiff > 0) {
			azimuth += TurnRate;
		} else {
			azimuth -= TurnRate;
		}
		normaliseAngle(azimuth);
	}
	updateHurtFlash();
}

void Turret::doFire()
{
	fireTimer = fireSpeed;
	Vect3d shotPos = Vect3d::fromPolar(azimuth, radius);
	shotPos = shotPos + location;
	Vect3d shotVelocity = Vect3d::fromPolar(azimuth, ShotSpeed);
	shotVelocity = shotVelocity;
	Shot * shot = new Shot(manager, this);
	shot->setLocation(shotPos);
	shot->setVelocity(shotVelocity);
	manager.addObject(shot);
}

void Turret::takeDamage(double amount)
{
	hurtFlash = 1;
	hitPoints -= amount;
	if (hitPoints <0) {
		dead = 1;
		// Create an explosion.
		Explosion *exp = new Explosion(manager);
		exp->setLocation(location + Vect3d(0,0,0.25));
		manager.addObject(exp);
		manager.playerInfo.score += 25;
		if (type == MapBlock::TurretItem3) {
			// give them 25 more for a smart turret.
			manager.playerInfo.score += 25;
		}
		manager.playerInfo.needsUpdate = true;
	}
}

/* Saucer */

Saucer::Saucer(Manager &m, MapBlock::MapItemType t) : Enemy(m), type(t) 
{
	location.z = SaucerHeight;
	collidable = true;
	fireTimer = MaxFireTime;
	fireSpeed = MaxFireTime;
	radius = 0.5;
	hitPoints = 250;
}

void Saucer::setLocation(const Vect3d &where)
{ 
	location = where; 
	location.z = SaucerHeight;
}

			
void Saucer::render()
{
	glEnable(GL_LIGHTING);
	glDisable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glColor3f(1,1,1);
	manager.rm.saucertexture.Bind();
	glPushMatrix();
	float scale = 1.0 / 12;
	glTranslatef(location.x, location.y, location.z);
	glScalef(scale,scale,scale);
	glRotatef(- azimuth, 0.0f, 0.0f, 1.0f);
	manager.rm.saucermodel.draw(true);
	glPopMatrix();
	drawHurtFlash();
}

void Saucer::runLogic()
{
	checkBounds();
	if (type == MapBlock::SaucerItem1) {
		azimuth += SaucerTurnRate;
	} else {
		azimuth -= SaucerTurnRate;
	}
	normaliseAngle(azimuth);
	if (fireTimer > 0) {
		fireTimer --;
	} else {
		// Determine if we're close to the player
		Vect3d rel = manager.playerLocation - location;
		double playerAngle = rel.angle();
		double azimuthDiff = playerAngle - azimuth;
		normaliseAngle(azimuthDiff);
		if (fabs(azimuthDiff) < 5.0) {
			doFire();
		}
	}
	updateHurtFlash();
}

void Saucer::doFire()
{
	fireTimer = fireSpeed;
	Vect3d shotPos = Vect3d::fromPolar(azimuth, radius);
	shotPos = shotPos + location;
	Vect3d shotVelocity = Vect3d::fromPolar(azimuth, ShotSpeed);
	shotVelocity = shotVelocity;
	Shot * shot = new Shot(manager, this);
	shot->setLocation(shotPos);
	shot->setVelocity(shotVelocity);
	manager.addObject(shot);
}

void Saucer::takeDamage(double amount)
{
	hurtFlash = 1;
	hitPoints -= amount;
	if (hitPoints <0) {
		dead = 1;
		// Create an explosion.
		Explosion *exp = new Explosion(manager);
		exp->setLocation(location + Vect3d(0,0,0.25));
		exp->setRadius(radius);
		manager.addObject(exp);
		manager.playerInfo.score += 100;
		manager.playerInfo.needsUpdate = true;
	}
}

