#ifndef GAMEOBJECT_INCLUDED
	#define GAMEOBJECT_INCLUDED

#include "vector3d.h"
#include "manager.h"
	
class GameObject {
	protected:
		Vect3d location;
		Vect3d velocity;
		double azimuth;
		double radius;
		bool dead;
		bool collidable;
		bool projectile;
		Manager & manager;
		double hurtFlash;
		
	public:
		double hitPoints;
		GameObject(Manager &m) : velocity(Vect3d(0,0,0)), azimuth(0), radius(1.0), 
			dead(false), collidable(false), projectile(false), manager(m), 
			hurtFlash(0), hitPoints(100)  { }
		Vect3d getLocation() { return location; }
		virtual void setLocation(const Vect3d &where) { location = where; }
		double getRadius() { return radius; }
		bool isCollidable() { return collidable; }
		bool isProjectile() { return projectile; }
		bool isDead() { return dead; }
		virtual void render() = 0;
		virtual void runLogic() = 0;
		virtual ~GameObject() { }
		void clip();
		virtual void handleMapCollision(const Vect3d & where);
		void checkBounds();
		virtual void handleCollision(GameObject *other) { }
		virtual void takeDamage(double amount) { hurtFlash = 1; }
		void drawHurtFlash();
		void updateHurtFlash();

	private:
		bool testLocation(const Vect3d & where);
		void testBlock(int ix, int iy, double x, double y);
};

class Explosion : public GameObject {
	private:
		int timer;
		static const int NumFrames = 6;
		static const int FrameLength = 5;
	public:
		Explosion(Manager &m) : GameObject(m), timer(0) {
			radius = 0.5;
		}
		virtual void render();
		virtual void runLogic();
		virtual ~Explosion() { }	
		void setRadius(double r) { radius = r; }
};

		
#endif //GAMEOBJECT_INCLUDED

