#include "graphics.h"

#include <GL/gl.h>
#include "SDL_ttf.h"

// This is intended to be drawn with blending.

void DrawLuminousQuad(Vect3d centre, Component size, 
	Component r, Component g, Component b)
{
	glBegin(GL_TRIANGLE_FAN);
	glColor4f(r,g,b,0);
	glVertex3f(centre.x, centre.y, centre.z);
	glColor4f(0,0,0,1);
	glVertex3f(centre.x + size, centre.y, centre.z);
	glVertex3f(centre.x, centre.y + size, centre.z);
	glVertex3f(centre.x - size, centre.y, centre.z);
	glVertex3f(centre.x, centre.y - size, centre.z);
	glVertex3f(centre.x + size, centre.y, centre.z);
	glEnd();
}

const int SurfaceWidth = 400;
const int SurfaceHeight = 32;

static SDL_Surface *surface = 0;
static TTF_Font *font;


/* This draws some stuff on to the screen or something */
void DrawTextString(Vect3d centre)
{
	glPixelZoom( 1.0, -1.0);
	glRasterPos3f(centre.x, centre.y, centre.z);
	glDrawPixels( SurfaceWidth, SurfaceHeight, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels );
}

void UpdateTextString(const char *str)
{
	SDL_Color white;
	white.r = 255;
	white.g = 255;
	white.b = 255;
	SDL_Color black;
	black.r = 0;
	black.g = 0;
	black.b = 0;
	
	SDL_Surface *tempsurf = 
		TTF_RenderText_Solid(font, str, white);
	SDL_Rect wholesurf;
	wholesurf.x = 0;
	wholesurf.y = 0;
	wholesurf.w = surface->w;
	wholesurf.h = surface->h;
	SDL_FillRect(surface, & wholesurf, 0);
	SDL_Rect rect;
	rect.x = rect.y = 0;
	rect.w = tempsurf->w;
	rect.h = tempsurf->h;
	SDL_BlitSurface(tempsurf, &rect, surface, &rect);
	SDL_FreeSurface(tempsurf);
}

void InitGraphics()
{
	if(TTF_Init()==-1) {
		throw "TTF init failed";
	}
	
	font = TTF_OpenFont("fonts/animeace.ttf", 18);
	if (font == 0) {
		throw "Failed to load font";
	}
	
	Uint32 rmask, gmask, bmask;
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	rmask = 0xff000000;
	gmask = 0x00ff0000;
	bmask = 0x0000ff00;
#else
	rmask = 0x000000ff;
	gmask = 0x0000ff00;
	bmask = 0x00ff0000;
#endif
	Uint32 width, height;
	width = SurfaceWidth;
	height = SurfaceHeight;
	
	surface = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, 24,
                                   rmask, gmask, bmask, 0);	
	if (surface == 0) {
		throw "Can't create text surface";
	}
}

