#include "constants.h"
#include "vector3d.h"
#include "model.h"
#include "texture.h"
#include "timing.h"
#include "manager.h"
#include "graphics.h"

#include <cmath>
#include <cstdlib>
#include <iostream>
#include <GL/gl.h>
#include <GL/glu.h>

#include "SDL.h"

#define SWIDE 800
#define SHIGH 600

/*
 * World dimensions:
 *
 * Screen is 12 units wide and 9 high
 * Camera is 12 units high (z = 12)
 * Min depth (near z) is 6.0
 * Far depth (far z) is 18.0
 *
 */

bool keys[512]; // array of bools for the key pressed state
bool mousebuttons[8];

static int ScreenWidth;
static int ScreenHeight;

static GLfloat LightAmbient[]  = {0.2f, 0.2f, 0.2f, 1.0f};
static GLfloat LightDiffuse[]  = {0.8f, 0.8f, 0.8f, 1.0f};
static GLfloat LightPosition[] = {1.0f, 1.0f, 1.5f, 0.0f}; // last number 0 = Infinity distance in that dir

static Texture testtexture;
static Texture shiptexture;


static void ResetGlStuff()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);           // Clear The Screen And The Depth Buffer
	
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();                  // Reset The View
	gluLookAt(
		0,0,CameraHeight, // camera
		0,0,0, // lookat
		0,1,0); // up

	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);
	glLightfv(GL_LIGHT1, GL_POSITION, LightPosition);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHTING);
	glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);
	// glShadeModel(GL_SMOOTH);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_CULL_FACE);
	// glDisable(GL_CULL_FACE);
}

static bool paused;

	/*
static void DrawTestStuff(Manager &man)
{
	
	man.rm.shiptexture.Bind();
	glPushMatrix();
		float scale = 1.0 / 20;
		glTranslatef(0,1.0, 3.0);
		glScalef(scale,scale,scale);
		glRotatef(rotangle, 0.0f, 0.0f, 1.0f);
		float wobble = ((rotangle * 10) / 180.0) * M_PI;
		glRotatef(sin(wobble) * 20.0, 0.0f, 1.0f, 0.0f);
		man.rm.shipmodel.draw(true);
	glPopMatrix();
	// glEnable(GL_BLEND);
	// glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
}
	*/

static void BeginRender() { 
	ResetGlStuff();
        glEnable(GL_LIGHTING);
	// Ignore depth test for the ground.
	// glDisable(GL_DEPTH_TEST);
	glEnable(GL_DEPTH_TEST);
	// Draw the object.
	glMatrixMode(GL_MODELVIEW);
}

static void EndRender() {
	// swap buffers to display, since we're double buffered.
        SDL_GL_SwapBuffers ();
}

static void InitGL(int Width, int Height)              // We call this right after our OpenGL window is created.
{
	ScreenWidth = Width;
	ScreenHeight = Height;
	// glClearColor(0.0f, 0.0f, 0.2f, 0.0f);         // This Will Clear The Background Color To Blue
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);         // Clear black.
	glClearDepth(30.0);                            // Enables Clearing Of The Depth Buffer
	glDepthFunc(GL_LESS);                         // The Type Of Depth Test To Do
	glEnable(GL_DEPTH_TEST);                      // Enables Depth Testing
	// glShadeModel(GL_SMOOTH);                      // Enables Smooth Color Shading
	glEnable(GL_NORMALIZE); 	// Normalise our normals please.
	// Set up the projection matrix - we should ony have to do this once.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();                             // Reset The Projection Matrix
	
	float fovy = 180 * (atan(4.5 / CameraHeight) / M_PI);
	fovy *= 2;
	std::cout << "fov Y = " << fovy << std::endl;
	// Make the far plane really far away.
	gluPerspective(fovy,(GLfloat)ScreenWidth/(GLfloat)ScreenHeight,6.0f, 50.0f);
		// Calculate The Aspect Ratio Of The Window
	ResetGlStuff();
}

static bool HandleSDL (Manager &man, SDL_Event *event )
{
        bool done = false;

        switch ( event->type )
        {
                case SDL_QUIT:
                        done = true;
                        break;

                case SDL_KEYDOWN:
                        if ( event->key.keysym.sym == SDLK_ESCAPE ) done = 1;
                        if ( event->key.keysym.sym == SDLK_PAUSE ) paused = true;
                        if ( event->key.keysym.sym == SDLK_SPACE ) paused = false;
                        keys[event->key.keysym.sym] = true;
                        break;

                case SDL_KEYUP:
                        keys[event->key.keysym.sym] = false;
                        break;
		case SDL_MOUSEMOTION:
			{
				double x = (double) event->motion.x / SWIDE;
				double y = (double) event->motion.y / SHIGH;
				man.setMousePosition(x,y);
				break;
			}
		case SDL_MOUSEBUTTONDOWN:
		case SDL_MOUSEBUTTONUP:
			if (event->button.button < 8) {
				mousebuttons[event->button.button] = (event->button.state == SDL_PRESSED);
				man.setMouseButton(event->button.button, event->button.state);
			}
			break;
        }

        return done;
}

const int FrameLength = 20; // milliseconds

/*
static void TestLoop()
{
	bool done = false;
	unsigned int endTick = GetMilliseconds() + 500;
	while (! done) {
		unsigned int now = GetMilliseconds();
		cout << "now=" << now << endl;
		if (now > endTick) {
			done = true;
		}
	}
}
*/

static bool HandleIntermissionEvent (SDL_Event *event )
{
        switch ( event->type )
        {
                case SDL_QUIT:
                        return true;
                case SDL_KEYDOWN:
                        keys[event->key.keysym.sym] = true;
                        break;

                case SDL_KEYUP:
                        keys[event->key.keysym.sym] = false;
                        break;
        }
        return false;
}

static bool ShowIntermission(const char *msg)
{
	UpdateTextString(msg);
	bool finished = false;
	do {
		BeginRender();
		DrawTextString(Vect3d(0, 0,0));
		EndRender();
		SDL_Event event;
		while ( SDL_PollEvent ( &event ) )
		{
			finished = HandleIntermissionEvent(&event);
		}
		SDL_Delay(100); // milliseconds.
		if (keys[SDLK_ESCAPE]) {
			return false;
		}
		if (keys[SDLK_SPACE]) {
			return true;
		}
		if (keys[SDLK_RETURN]) {
			return true;
		}
	} while (! finished);
	return false;
}

static void MainLoop(Manager & manager)
{
	bool done = false;
	paused = false;
	unsigned int nextTick = GetMilliseconds() + 500;
	while (! done) {
		unsigned int now = GetMilliseconds();
		// cout << now << endl;
		// Busy-wait.
		while (nextTick > now) {
			SDL_Delay(10);
			now = GetMilliseconds();
		}
		
		if (! paused) {
			manager.runLogic();
		}
		
		nextTick += FrameLength;
		now = GetMilliseconds();
		if (nextTick > now) {
			BeginRender();
			manager.render();
			EndRender();
			// Handle any pending SDL events
			SDL_Event event;
			while ( SDL_PollEvent ( &event ) )
			{
				done = HandleSDL (manager,  &event );
			}
		}
		if (manager.isStateChangeReady()) {
			done = true;
		}
	}
}

static void RunGame(int startLevel)
{
	Manager manager;
	manager.init();
	manager.playerInfo.level = startLevel;
	do {
		char buf[200];
		sprintf(buf,"Entering level %d", manager.playerInfo.level);
		if (! ShowIntermission(buf)) {
			return;
		}
		manager.initLevel();
		MainLoop(manager);
		if (manager.isLevelComplete()) {
			manager.playerInfo.level ++;
		} else {
			manager.playerInfo.lives --;
		}
	} while ((manager.playerInfo.lives >0) && (manager.playerInfo.level < 7));
	// Game oer.
	if (manager.playerInfo.lives == 0) {
		ShowIntermission("Game over");
	} else {
		ShowIntermission("Well done, you win!");
	}
}

static void InitInput()
{
	for (int i=0; i<512; i++) {
		keys[i] = false;
	}
	for (int i=0; i<8; i++) {
		mousebuttons[i] = false;
	}
}

int main(int argc, char *argv[])
{
	bool fullscreen = false;
	int startLevel = 1;
	for (int i=1; i<argc; i++) {
		if (strcmp(argv[i], "f") == 0) {
			fullscreen = true;
		}
		if (argv[i][0] == 'l') {
			startLevel = argv[i][1] - '0';
		}
	}
	SDL_Init ( SDL_INIT_VIDEO);
	int flags = SDL_OPENGL;
	if (fullscreen) flags |= SDL_FULLSCREEN;
	flags |= SDL_DOUBLEBUF;
	SDL_GL_SetAttribute ( SDL_GL_RED_SIZE, 5 );
	SDL_GL_SetAttribute ( SDL_GL_GREEN_SIZE, 5 );
	SDL_GL_SetAttribute ( SDL_GL_BLUE_SIZE, 5 );
	if (! SDL_SetVideoMode ( SWIDE, SHIGH, 16, flags )) {
		std::cerr << "Unable to get video mode" << std::endl;
		return 1;
	}
	try {
		InitGraphics();
		InitGL(SWIDE,SHIGH);
		InitInput();
		InitMilliseconds();
		// TestLoop();
		RunGame(startLevel);
	} catch (const char * ex) {
		SDL_Quit();
		std::cerr << "Unexpected exception:" << ex << std::endl;
		return 1;
	} catch (...) {
		SDL_Quit();
		std::cerr << "Unknown type of exception" << std::endl;
		return 1;
	}
	SDL_Quit();

	return 0; // reached
}
