#ifndef MANAGER_INCLUDED
	#define MANAGER_INCLUDED

#include "resourcemanager.h"
#include "map.h"
#include <vector>
	
class GameObject;
class Manager;
class Ship;

class StarField {
	private:
		struct Star {
			Vect3d location;
			double brightness;
		};
		typedef std::vector<Star> StarList;
		StarList stars;
	public:
		StarField();
		void runLogic(Manager &m);
		void render();
		void init();
};

struct PlayerInfo {
	int score;
	int lives;
	int level;
	double hitPoints;
	bool needsUpdate;
	void reset() {
		score = 0;
		lives = 3;
		level = 1;
		needsUpdate = true;
	}
};

class Manager {
	private:
		typedef std::vector<GameObject *> GameObjectList;
		typedef GameObjectList::iterator GameObjectIterator;
		GameObjectList objects;
		GameObjectList newobjects;
		GameObjectList deadobjects;
		double scrollOffset;
		double mouseScreenx;
		double mouseScreeny;
		int maxblockx;
		StarField starfield;
		// Set if we want to do something soon.
		bool stateChangePending;
		// Time until we do something (end or restart the level)
		int stateChangeTimer;
		// Is player dead?
		bool playerDead;
		bool levelComplete;
	public:
		ResourceManager rm;
		PlayerInfo playerInfo;
		Manager() {
			
		}
		~Manager();
		void initLevel();
		
		void addObject(GameObject * obj);
		
		void init();
		void render();
		void runLogic();
		Vect3d mousePosition;
		Vect3d playerLocation;
		bool mousebuttons[8];
		void setMousePosition(double x, double y);
		void setMouseButton(int button, bool state) {
			mousebuttons[button] = state;
		}
		Map map;
		double getScrollOffset() {
			return scrollOffset;
		}
		double getLeftBoundary() {
			return (scrollOffset - 6);
		}
		double getTopBoundary() {
			return 11;
		}
		double getBottomBoundary() {
			return -2;
		}
		
		double getScrollSpeed();
		bool isLevelComplete() {
			return levelComplete;
		}
		bool isStateChangeReady() {
			return (stateChangePending && (stateChangeTimer <=0));
		}
	private:
		void deleteList(GameObjectList &list);
		void deleteAll();
		void renderBackground();
		void calcMousePosition();
		void createScrolledObjects();
		void collideDetect();
		void testCollision(GameObject * collidable, GameObject *projectile);
		Ship * playerShip;
		double scrollSpeed;
		void updatePlayerInfo();
		void drawHitPoints();
};

#endif //MANAGER_INCLUDED

