#include "ship.h"
#include "shot.h"
#include <GL/gl.h>
#include "SDL.h"

void Ship::render()
{
	glEnable(GL_LIGHTING);
	glDisable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glColor3f(1,1,1);
	manager.rm.shiptexture.Bind();
	// manager.rm.saucertexture.Bind();
	glPushMatrix();
	float scale = 1.0 / 16;
	glTranslatef(location.x, location.y, location.z);
	glScalef(scale,scale,scale);
	glRotatef(-azimuth, 0.0f, 0.0f, 1.0f);
	//float roll = ((30.0) / 180.0) * M_PI;
	// float roll = 0.0;
	glRotatef(roll, 0.0f, 1.0f, 0.0f);
	manager.rm.shipmodel.draw(true);
	glPopMatrix();
	drawHurtFlash();
}

static const double AzimuthTolerance = 4.0;
static const double TurnRate = 3.0;
static const double MaxSpeed = 0.06; // Units per tick ( = 1/50 s)
static const double MaxRoll = 45.0;
static const double RollRate = 2.0;
static const double MaxAcceleration = (MaxSpeed / 40);
static const double HpRechargeRate = 0.04;

void Ship::runLogic()
{
	Vect3d toPointer = manager.mousePosition - location;
	double desiredAzimuth = toDegrees(atan2(toPointer.x, toPointer.y));
	double azimuthDiff = desiredAzimuth - azimuth;
	normaliseAngle(azimuthDiff);
	double targetRoll = 0;
	if (fabs(azimuthDiff) < AzimuthTolerance) {
		azimuth = desiredAzimuth;
	} else {
		if (azimuthDiff > 0) {
			targetRoll = MaxRoll;
			azimuth += TurnRate;
		} else {
			targetRoll = - MaxRoll;
			azimuth -= TurnRate;
		}
		normaliseAngle(azimuth);
	}
	if (targetRoll > roll) {
		roll += RollRate;
	}
	if (targetRoll < roll) {
		roll -= RollRate;
	}
	if (fabs(roll - targetRoll) < RollRate) {
		roll = targetRoll;
	}
	double speed = MaxSpeed;
	if (! manager.mousebuttons[SDL_BUTTON_RIGHT]) {
		speed = speed / 4;
	}
	bool superBoost = false;
	if (location.x < (manager.getLeftBoundary() + 2)) {
		speed = MaxSpeed;
		if (manager.getScrollSpeed() > MaxSpeed) {
			// We're obviously zipping along here.
			speed = manager.getScrollSpeed() * 1.25;
			superBoost = true;
		}
	}
	Vect3d targetVelocity = Vect3d::fromPolar(azimuth, speed);
	if (superBoost) {
		velocity = targetVelocity;
	} else {
		approachVelocity(targetVelocity);
	}
	location = location + velocity;
	clip();
	if (manager.mousebuttons[SDL_BUTTON_LEFT] && (fireTimer == 0)) {
		doFire();
	} else {
		if (fireTimer > 0) {
			fireTimer --;
		} else {
			// They're not firing...
			hitPoints += HpRechargeRate;
			if (hitPoints > 100) hitPoints = 100;
		}
	}
	updateHurtFlash();
	if (location.x < (manager.getLeftBoundary() - 2)) {
		hitPoints = hitPoints - 1;
		if (hitPoints <0) {
			dead = true;
		}
	}
}

static const double ShotInitialDistance = 0.5;
static const double ShotSpeed = 0.1;

void Ship::doFire()
{
	fireTimer = fireSpeed;
	Vect3d shotPos = Vect3d::fromPolar(azimuth, ShotInitialDistance);
	shotPos = shotPos + location;
	Vect3d shotVelocity = Vect3d::fromPolar(azimuth, ShotSpeed);
	shotVelocity = shotVelocity + velocity;
	Shot * shot = new Shot(manager, this);
	shot->setLocation(shotPos);
	shot->setVelocity(shotVelocity);
	manager.addObject(shot);
}

void Ship::approachVelocity(const Vect3d & targetVelocity)
{
	Vect3d diff = targetVelocity - velocity;
	if (diff.magnitudeSquared() < (MaxAcceleration * MaxAcceleration)) {
		velocity = targetVelocity;
	} else {
		diff.normalise();
		diff = diff * MaxAcceleration;
		velocity = velocity + diff;
	}
}

void Ship::takeDamage(double amount)
{
	hurtFlash = 1;
	hitPoints -= amount;
	if (hitPoints <0) {
		dead = 1;
		// Create an explosion.
		Explosion *exp = new Explosion(manager);
		exp->setLocation(location + Vect3d(0,0,0.25));
		exp->setRadius(0.75);
		manager.addObject(exp);
	}
}
